#! /usr/bin/env python3
import os
import sys
import argparse
import logging
import logging.handlers
from pyroute2 import IPDB
from pyroute2 import NetNS
from pyroute2.netlink.exceptions import NetlinkError
import sysrepo_interfaces.sysrepo_interfaces3 as sri
import vppapi_if.vppapi_if as vppif

log = None

def set_log( name, level ):
	global log
	log = logging.getLogger(name=name)
	log.setLevel(level=level)

	syslog_hnd = logging.handlers.SysLogHandler(address = '/dev/log')
	formatter = logging.Formatter('%(name)s: %(levelname)s %(message)s')
	syslog_hnd.setFormatter(formatter)
	log.addHandler(syslog_hnd)


def clear_dhcpd_cfg(ifname, pid_file, leases_file, conf_file):
    if os.path.exists(pid_file):
        try:
            pid = open(pid_file, 'rb').readline()
            os.kill(int(pid), 9)
            os.remove(pid_file)
        except:
            log.debug('There is no {}-dhcpd process'.format(ifname))
            
    if os.path.exists(leases_file):
        log.debug('Delete {}'.format(leases_file))
        os.system('rm -rf {}*'.format(leases_file))

    if os.path.exists(conf_file):
        log.debug('Delete {}'.format(conf_file))
        os.remove(conf_file)


def dhcpd_cfg(ifname, local_ip, remote_ip, ldns, is_add):
    conf_file = '/var/{}-dhcpd.conf'.format(ifname)
    leases_file = '/var/lib/dhcp/ns{}.leases'.format(ifname)
    nsifname = 'ns{}'.format(ifname)
    nsname = '{}_dhcpd'.format(ifname)
    pid_file = '/var/run/{}-dhcpd.pid'.format(ifname)

    clear_dhcpd_cfg(ifname, pid_file, leases_file, conf_file)

    if is_add and ldns is not None:
        os.system('/usr/bin/touch {}'.format(leases_file))
        os.system('/usr/bin/touch {}'.format(pid_file))
        cfg = 'option domain-name-servers {};\n'.format(ldns)
        cfg += 'option subnet-mask 255.255.255.255;\n'
        cfg += 'option routers {};\n'.format(remote_ip)
        cfg += 'shared-network "{}" {{\n'.format(nsifname)
        cfg += '    subnet {} netmask 255.255.255.255 {{\n'.format(remote_ip)
        cfg += '    }\n'
        cfg += '    subnet {} netmask 255.255.255.255 {{\n'.format(local_ip)
        cfg += '        range {};\n'.format(local_ip)
        cfg += '    }\n}\n'
        cfg += 'lease-file-name "/var/lib/dhcp/{}.leases";\n'.format(nsifname)
        cfg += 'deny declines;\ndeny duplicates;\none-lease-per-client true;\nignore-client-uids true;\nauthoritative;'\
                '\nallow unknown-clients;\nping-check false;'

        with open(conf_file, "w") as cfg_file:
            log.debug('Write to {}'.format(conf_file))
            cfg_file.write(cfg)
        try:
            log.debug('Run {}-dhcpd server on {}'.format(ifname, nsifname))
            os.system('ip netns exec {} dhcpd -cf {} -pf {}'.format(nsname, conf_file, pid_file))
        except:
            log.debug('Could not run {}-dhcpd server'.format(ifname))

    if not is_add:
        log.debug('Clear dhcpd-{} config'.format(ifname))
        clear_dhcpd_cfg(ifname, pid_file, leases_file, conf_file)

def vpp_ippass_nsif_cfg(ifname, local_ip, remote_ip, is_add):
	nsname = '{}_dhcpd'.format(ifname)
	nsifname = 'ns{}' .format(ifname)
	ip_prefix = '{}/32' .format(remote_ip)

	ipdb = IPDB(nl=NetNS(nsname))
	if nsifname in ipdb.interfaces:
		nsif_idx = ipdb.interfaces[nsifname]
		if is_add == True:
			nsif_idx.add_ip(ip_prefix)
			nsif_idx.up()
			ipdb.commit()
		else:
			nsif_idx.del_ip(ip_prefix)
			nsif_idx.down()
			ipdb.commit()

	ipdb.release()

def vpp_bd_ip_pass_cfg(ifname, local_ip, remote_ip, is_add):

	vpp_if = vppif.VppApiInterface('ppp_ippass_cfg')
	(if_name, if_idx) = vpp_if.search_vppif_by_hostifname(ifname)
	if if_name == None or if_idx == None:
		del(vpp_if)
		return

	if is_add == False:
		vpp_if.flush_bd_ip_mac_entries(bd_id=if_idx)
		log.debug('Static ARP Term entries are removed from interface {}' .format(ifname))
		del(vpp_if)
		return

	nstap = vpp_if.get_tap_interface_by_host_ifname('ns{}' .format(ifname))
	if not nstap:
		del(vpp_if)
		return
	vpp_if.add_bd_ip_mac_entry(bd_id=if_idx, ip=remote_ip, mac=nstap['host_mac_addr'])
	log.debug('Added static ARP Term entry {} / {} for interface {}'.format(nstap['host_mac_addr'], remote_ip, ifname))
	del(vpp_if)
	return 0

def pppif_ip_pass_cfg(ifname, local_ip, remote_ip, is_add=True, ldns=None):
	if not ifname:
		return
	try:
		ippass = srcl.get_interface_param(ifname, 'ip-passthrough')
		if ippass != True:
			log.debug('IP-Passthrough not enabled for interface {}' .format(ifname))
			return

		# Apply ip-pass management entries
		srcl.ippass_add_del_management_entries(ifname, local_ip, is_add)
		# ARP Term entry
		vpp_bd_ip_pass_cfg(ifname, local_ip, remote_ip, is_add)
		# nspppX interface config
		vpp_ippass_nsif_cfg(ifname, local_ip, remote_ip, is_add)
		# NS DHCPD config
		dhcpd_cfg(ifname, local_ip, remote_ip, ldns, is_add)
	except:
		log.error('PPPoE IP-Passthrough config not applied!')


def set_nat_wan_route(natifname, pppifname):
	rtindex_offset = 1
	table_id = int(natifname.replace('nat', '')) + rtindex_offset

	ipdb = IPDB()
	if pppifname in ipdb.interfaces:
		wan_idx = ipdb.interfaces[pppifname]
		for rt in ipdb.routes:
			if rt['family'] == 10:
				continue
			if rt['oif'] == wan_idx['index']:
				if rt['gateway']:
					route = {'dst': rt['dst'], 'gateway': rt['gateway'], 'table': table_id, 'priority': rt['priority']}
				else:
					route = {'dst': rt['dst'], 'table': table_id, 'oif': wan_idx['index']}
				
				try:
					ipdb.routes.add(route).commit()
				except NetlinkError as e:
					if e.code != 17:
						log.error('Set route in table {tbl} failed. {msg}'.format(tbl=table_id, msg=str(e)))
	ipdb.release()

def pppif_nat_cfg(ifname):
	if not ifname:
		return
	try:
		startup = srcl.get_startup_config(full_path=False)
		nats = list(filter(lambda x: True if x.get('type', '') == 'nat' 
				and ( x.get('enable', False) == True or x.get('enable', False) == 'true' ) 
				and x.get('wan-interface', None) != None else False, startup))
		if not nats:
			return
		
		ppp_nats = list(filter(lambda elem: elem.get('wan-interface', None) is not None and ifname in [x['name'] for x in elem.get('wan-interface', None)], nats))
		
		for natif in ppp_nats:
			set_nat_wan_route(natif.get('name', None), ifname)
	except:
		log.error('NAT pppoe config not applied!')

if __name__ == "__main__":
	
	set_log( 'pppif-cfg', logging.DEBUG )
	parser = argparse.ArgumentParser()
	
	parser.add_argument("-n", "--name", required=True, type=str, help="PPPoE interface name to manipulate.")
	parser.add_argument("-l", "--local-ip", type=str, help="Local peer IP Address.")
	parser.add_argument("-r", "--remote-ip", type=str, help="Remote peer IP Address.")
	parser.add_argument("-s", "--ldns", type=str, help="List of DNSs.")
	parser.add_argument('-d', dest='delete', action='store_true', default=False)
	args = parser.parse_args()

	srcl = sri.SysrepoInterfaces(app_name='pppif-cfg',priority=2)

	if args.local_ip and args.remote_ip:
		log.debug('Interface {} - Set local IP {} and remote IP {}'.format(args.name, args.local_ip, args.remote_ip))
		pppif_nat_cfg(args.name)
		pppif_ip_pass_cfg(args.name, args.local_ip, args.remote_ip, True, args.ldns)

		if args.delete:
			log.debug('Interface {} - Remove configuration'.format(args.name))
			pppif_ip_pass_cfg(args.name, args.local_ip, args.remote_ip, False, None)
			sys.exit(0)

	sys.exit(0)
