#! /usr/bin/env python3
import os
import sys
import argparse
import shutil
import subprocess
from systemd import journal
from configobj import ConfigObj

switch_net_mode_stream = journal.stream(identifier='switch-ovs-vpp')

ovs_s = ['ovs-config.service', 'openvswitch-switch.service', 'ovsdb-server.service', 'ovs-vswitchd.service', \
                               'ippass.service']

vpp_s = ['vpp-config.service', 'vpp_events.service', 'netlink-vpp.service']

def update_collectd_config(tovpp):
    if tovpp is True:
        if os.path.isfile('/etc/collectd/collectd.conf.d/ovs.conf'):
            os.remove('/etc/collectd/collectd.conf.d/ovs.conf')

        if os.path.isfile('/opt/etc/vpp.conf'):
            shutil.copyfile('/opt/etc/vpp.conf', '/etc/collectd/collectd.conf.d/vpp.conf')
    else:
        if os.path.isfile('/etc/collectd/collectd.conf.d/vpp.conf'):
            os.remove('/etc/collectd/collectd.conf.d/vpp.conf')

        if os.path.isfile('/opt/etc/ovs.conf'):
            shutil.copyfile('/opt/etc/ovs.conf', '/etc/collectd/collectd.conf.d/ovs.conf')

def is_masked(svc):
    try:
        state = subprocess.check_output(['systemctl', 'is-enabled', svc ])
        if state.decode().rstrip() == 'masked':
            return True
    except subprocess.CalledProcessError as e:
        if e.output.decode().rstrip() == 'masked':
            return True

    return False

def manage_services(action, svcs):
    try:
        for svc in svcs:
            # subprocess.call(['systemctl', action, svc], stdout=subprocess.DEVNULL, stderr=subprocess.PIPE)
            subprocess.call(['systemctl', action, svc], stderr=subprocess.PIPE)
    except subprocess.CalledProcessError as e:
        print("Command \"{}\" failed with code {}".format(e.cmd, e.returncode))
    except OSError as e:
        print("System error - \"{}\"".format(e.strerror))

def to_ovs():
    if is_masked(vpp_s[0]) is True:
        sys.exit("OVS is already set as network mode")
    update_collectd_config(tovpp=False)
    manage_services(action='disable', svcs=vpp_s)
    manage_services(action='mask', svcs=vpp_s)
    manage_services(action='unmask', svcs=ovs_s)
    manage_services(action='enable', svcs=ovs_s)
    del_vxlan_ifcfg_files()

def to_vpp():
    if is_masked(ovs_s[0]) is True:
        sys.exit("VPP is already set as network mode")
    vpp_svs = vpp_s.copy()

    update_collectd_config(tovpp=True)
    manage_services(action='disable', svcs=ovs_s)
    manage_services(action='mask', svcs=ovs_s)
    manage_services(action='unmask', svcs=vpp_s)
    manage_services(action='enable', svcs=vpp_svs)

def check_mode():
    if is_masked(ovs_s[0]) is True:
        mode = 'VPP'
        svcs = vpp_s
    else:
        mode = 'OVS'
        svcs = ovs_s
        print('Network Mode: {}\n'.format(mode))
        for svc in svcs:
            subprocess.call(['systemctl', 'status', svc ])

def enable_serives(mode, reboot):
    current_vpp_mode = is_masked(ovs_s[0])
    if 'ovs' == mode:
        update_collectd_config(tovpp=False)
        manage_services(action='disable', svcs=vpp_s)
        manage_services(action='mask', svcs=vpp_s)
        manage_services(action='unmask', svcs=ovs_s)
        manage_services(action='enable', svcs=ovs_s)
        if current_vpp_mode and reboot:
            system_reboot()
    elif 'vpp' == mode:
        update_collectd_config(tovpp=True)
        manage_services(action='disable', svcs=ovs_s)
        manage_services(action='mask', svcs=ovs_s)
        manage_services(action='unmask', svcs=vpp_s)
        manage_services(action='enable', svcs=vpp_s)
        if not current_vpp_mode and reboot:
            system_reboot()
            

def system_reboot():
    print("requested system reboot for network mode change", file=switch_net_mode_stream)
    if os.path.exists('/config/_reboot/tve/now'):
        os.rmdir('/config/_reboot/tve/now')

	# Create dirs
    os.makedirs('/config/_reboot/tve/now')

def del_vxlan_ifcfg_files():
    subprocess.call('rm /etc/network/interfaces.d/*vxlan* 2>/dev/null || true', shell=True)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Switch between OVS and VPP networking mechanisms')
    parser.add_argument('--to', type=str, choices=['ovs', 'vpp'], help="Switch to OVS or VPP mode")
    parser.add_argument('--reboot', '-r', action='store_true', help='Reboot uCPE after changing the mechanism')
    parser.add_argument('--check-mode', '-ch',action='store_true', help='Check current mode')
    parser.add_argument('--enable', type=str, choices=['ovs', 'vpp'], help="Enable services at startup")
    args = parser.parse_args()

    if args.check_mode is True:
        check_mode()
    elif args.enable == 'ovs' or args.enable == 'vpp':
        enable_serives(args.enable, args.reboot)
    else:
        if 'ovs' == args.to:
            to_ovs()
        elif 'vpp' == args.to:
            to_vpp()
        if args.reboot is True:
            system_reboot()
    sys.exit(0)
